% This script demos a bug that I discovered, and fixed, in the latest 
% 'official' version of ARPACK, namely the initial release plus patch 
% obtained from here:
%
% http://www.caam.rice.edu/software/ARPACK/
% 
% The requested eigenvalues from ARPACK can be wrong.  The problem
% occurs when the following two conditions both hold:
%
% 1) eigenvectors are requested (rvec = .true.) 
% 2) the resulting number of converged eigenvalues is less than the 
%   number requested.
%
% First, this file should be supplied with a test matrix file: 'testA.mat'.
%
% If you see different eigenvalues for the two calls to EIGS, then
% you need to patch your ARPACK code.  
%
% The patch is currently available from myself and is also in the latest 
% versions of both arpack-ng and Andreas Kloeckner's ARPACK distributions. 
%
% March, 6th, 2012
%
% Tim Mitchell
% tim.mitchell@cims.nyu.edu


% load example matrix rand(50,50)
load('testA.mat');

k = 10;
sigma = 'SR';
opts.maxit = 30; % make sure not all k Ritz values converge
opts.v0 = ones(length(A),1);

d = eigs(A,k,sigma,opts);
[V,D] = eigs(A,k,sigma,opts);

fprintf('EIGS, only eigenvalues requested:\n');
d
fprintf('EIGS, eigenvalues and eigenvectors both requested:\n');
%d = diag(D)
D

for j=1:length(d)
    if d(j) ~= 0
        fprintf('%g\n', norm(A*V(:,j) - V(:,j)*d(j)));
    end
end
